"""This module handles the global authentication.

Authentication is a global state.
When authenticating make sure to encrypt the connection.
All credentials can be read.
"""

from __future__ import absolute_import

from .configuration import Configuration
from base64 import b64encode


_config = Configuration()

def none():
    """Delete all authentication.

    This is the default case.
    """
    _config.username = None
    _config.password = None
    _config.api_key_prefix = {}
    _config.api_key = {}

    
def basic(username, password):
    """Add basic authentication to the requests of the clients."""
    none()
    _config.username = username
    _config.password = password


def api_key(api_key):
    """Authenticate via an api key."""
    none()
    _config.api_key_prefix["Authorization"] = "api-key"
    _config.api_key["Authorization"] = "key=" + b64encode(api_key.encode()).decode()

none()

__all__ = ["none", "basic", "api_key"]


# coding: utf-8

"""
    Schul-Cloud Content API

"""


from __future__ import absolute_import

import os
import sys
import unittest

import schul_cloud_ressources_api_v1
from schul_cloud_ressources_api_v1.schema import (
    validate_ressource, is_valid_ressource, get_valid_examples,
    get_invalid_examples, ValidationFailed
)


class TestSchema(unittest.TestCase):
    """ Schema unit test stubs """

    def test_there_are_valid_examples(self):
        self.assertTrue(get_valid_examples())

    def test_there_are_invalid_examples(self):
        self.assertTrue(get_valid_examples())

    def test_valid_and_invalid_examples_are_not_the_same(self):
        for v in get_valid_examples():
            for i in get_invalid_examples():
                self.assertNotEqual(v, i, "A valid example can not be invalid.")

    def test_all_valid_examples_succeed(self):
        for v in get_valid_examples():
            self.assertTrue(is_valid_ressource(v))
            validate_ressource(v)

    def test_all_invalid_examples_fail(self):
        for i in get_invalid_examples():
            self.assertFalse(is_valid_ressource(i))
            self.assertRaises(ValidationFailed, lambda: validate_ressource(i))


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
Tests for schul_cloud_ressources_api_v1.auth
"""


from __future__ import absolute_import

import os
import sys
import unittest
import base64


import schul_cloud_ressources_api_v1.auth as auth
from schul_cloud_ressources_api_v1.configuration import Configuration


class TestAuthentication(unittest.TestCase):
    """Test the authentication mechanism."""
    
    def setUp(self):
        """Create the default state.

        We have global state so we want to remove the side effects.
        """
        auth.none()
        self.config = Configuration()
    tearDown = setUp

    def test_no_authentication_has_no_user_name_and_password(self):
        """By default there is no username and poassword supplied."""
        self.assertFalse(self.config.username)
        self.assertFalse(self.config.password)

    def test_no_authentication_has_no_api_key(self):
        """"By default, the api key is not present."""
        self.assertFalse(self.config.api_key)
        self.assertFalse(self.config.api_key_prefix)


    def test_none_deletes_user_name_and_password(self):
        """After having used basic authentication, credentials can be deleted."""
        auth.basic("username", "password")
        auth.none()
        self.test_no_authentication_has_no_user_name_and_password()

    def test_basic_authentication_provides_username_and_password(self):
        for username, password in [("a", "p"), ("1", "2")]:
            auth.basic(username, password)
            self.assertIsBasic(username, password)

    def assertIsBasic(self, username, password):
        """Make sure we have basic authentication."""
        self.assertEqual(self.config.username, username)
        self.assertEqual(self.config.password, password)

    def test_api_key(self):
        """Test that the api key creates the correct values."""
        for key in ["api-key", "test", "jfsdgfghdsafdskhgfkadsgkgfkasd"]:
            auth.api_key(key)
            self.assertIsApiKey(key)

    def assertIsApiKey(self, key):
        """Make sure we have api key authentication."""
        settings = self.config.auth_settings()["api_key"]
        self.assertEqual(settings["type"], "api_key")
        self.assertEqual(settings["in"],"header")
        self.assertEqual(settings["key"],"Authorization")
        self.assertEqual(settings["value"],"api-key key=" + base64.b64encode(key.encode()).decode())

    def test_none_deletes_api_key(self):
        """Test deletion of api key."""
        auth.api_key("key")
        auth.none()
        self.test_no_authentication_has_no_api_key()

    def test_basic_removes_api_key(self):
        """The basic auth replaces the api key.

        Else, we might run into troubles with what is used last.
        """
        auth.api_key("key")
        auth.basic("user", "pass")
        self.test_no_authentication_has_no_api_key()

    def test_api_key_removes_basic(self):
        """The api key auth replaces the basic auth.

        Else, we might run into troubles with what is used last.
        """
        auth.basic("user", "pass")
        auth.api_key("key")
        self.test_no_authentication_has_no_user_name_and_password()


if __name__ == '__main__':
    unittest.main()



import jsonschema
import os
import json


HERE = os.path.dirname(__file__)
VALID_SCHEMAS_PATH = os.path.join(HERE, "json/ressource/examples/valid/")
INVALID_SCHEMAS_PATH = os.path.join(HERE, "json/ressource/examples/invalid/")
JSON_SCHEMA_PATH = os.path.join(HERE, "json/ressource/ressource.json")


ValidationFailed = jsonschema.exceptions.ValidationError


def get_ressource_schema():
    """Return the json schema for the ressource."""
    with open(JSON_SCHEMA_PATH, "rb") as file:
        schema = json.loads(file.read().decode("UTF-8"))
    schema["id"] = "file://" + JSON_SCHEMA_PATH
    return schema


def validate_ressource(ressource):
    """Validate the schema of a ressource.
    
    This function just passes if the schema matches the ressource.
    If the ressource does not fit, an ValidationFailed is raised.
    """
    jsonschema.validate(ressource, get_ressource_schema())


def is_valid_ressource(ressource):
    """Return whether a the given ressources fits into the schema."""
    try:
        validate_ressource(ressource)
    except ValidationFailed:
        return False
    return True


def _get_json_content_from_folder(folder):
    """yield objects from json files in the folder and subfolders."""
    for dirpath, dirnames, filenames in os.walk(folder):
        for filename in filenames:
            if filename.lower().endswith(".json"):
                filepath = os.path.join(dirpath, filename)
                with open(filepath, "rb") as file:
                    yield json.loads(file.read().decode("UTF-8"))


def get_valid_examples():
    """Return a list of json objects that are valid examples for ressources."""
    return list(_get_json_content_from_folder(VALID_SCHEMAS_PATH))


def get_invalid_examples():
    """Return a list of json objects that are invalid examples for ressources."""
    return list(_get_json_content_from_folder(INVALID_SCHEMAS_PATH))


__all__ = ["ValidationFailed", "validate_ressource", "is_valid_ressource",
           "get_valid_examples", "get_invalid_examples"]
